package domain

import (
	"cicv-data-closedloop/amd64/dispatch_server/package/entity"
	"cicv-data-closedloop/amd64/dispatch_server/package/global"
	"cicv-data-closedloop/amd64/dispatch_server/package/infra"
	"cicv-data-closedloop/amd64/dispatch_server/package/util"
	"encoding/json"
	"errors"
)

func CanRun(userId string, userParallelism int64) bool {
	TaskQueueRunningUserNumber, _ := infra.GlobalRedisClient.LLen(global.KeyTaskQueueRunningUserPrefix + ":" + userId).Result()
	if TaskQueueRunningUserNumber < userParallelism {
		return true
	}
	return false
}

func AddWaitingUser(userId string, userParallelism int64, task entity.Task) error {
	taskCacheTemp := entity.TaskCache{
		UserId:          userId,
		UserParallelism: userParallelism,
		Task:            task,
	}
	// 转 json
	taskCacheJson, err := TaskCacheToJson(taskCacheTemp)
	if err != nil {
		return errors.New("任务缓存对象" + util.ToString(taskCacheTemp) + " 转 json 失败，错误信息为: " + util.ToString(err))
	}
	// 添加到用户等待队列
	key := global.KeyTaskQueueWaitingUser
	err = infra.GlobalRedisClient.RPush(key, taskCacheJson).Err()
	if err != nil {
		return errors.New("任务缓存对象json " + taskCacheJson + "  " + key + " 失败，错误信息为: " + util.ToString(err))
	}
	return nil
}

func DeleteWaitingUser(taskCacheJson string) error {
	// 定义要删除的列表和值
	key := global.KeyTaskQueueWaitingUser
	value := taskCacheJson
	// 使用 LREM 命令删除列表中的特定值
	_, err := infra.GlobalRedisClient.LRem(key, 0, value).Result()
	if err != nil {
		return errors.New("删除用户等待队列中的值 " + taskCacheJson + " 失败，错误信息为: " + util.ToString(err))
	}
	return nil
}

func AddWaitingCluster(userId string, userParallelism int64, task entity.Task) error {
	taskCacheTemp := entity.TaskCache{
		UserId:          userId,
		UserParallelism: userParallelism,
		Task:            task,
	}
	// 转 json
	taskCacheJson, err := TaskCacheToJson(taskCacheTemp)
	if err != nil {
		return errors.New("任务缓存对象" + util.ToString(taskCacheTemp) + " 转 json 失败，错误信息为: " + util.ToString(err))
	}
	// 添加到集群等待队列
	err = infra.GlobalRedisClient.RPush(global.KeyTaskQueueWaitingCluster, taskCacheJson).Err()
	if err != nil {
		return errors.New("任务缓存对象json " + taskCacheJson + " 添加到集群等待队列失败，错误信息为: " + util.ToString(err))
	}
	return nil
}

func TaskToJson(task entity.Task) (string, error) {
	jsonData, err := json.MarshalIndent(task, "", "    ")
	if err != nil {
		return "", err
	}
	return string(jsonData), nil
}

func TaskCacheToJson(taskCache entity.TaskCache) (string, error) {
	jsonData, err := json.MarshalIndent(taskCache, "", "    ")
	if err != nil {
		return "", err
	}
	return string(jsonData), nil
}
